/**
 * $Id$
 */
package ftc;

import java.io.File;
import java.io.IOException;
import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;
import java.util.List;
import java.util.logging.Logger;

import org.kohsuke.args4j.CmdLineException;
import org.kohsuke.args4j.CmdLineParser;

import sun.reflect.generics.reflectiveObjects.NotImplementedException;
import tecgraf.ftc_1_2.common.exception.InvalidArraySize;
import tecgraf.ftc_1_2.server.MaxChannelRequestsException;
import ftc.server.v1_1.FTCServerV1_1;
import ftc.server.v1_2.FTCServerV1_2;
import ftc.server.v1_3.FTCServerV1_3;
import ftc.server.v1_4_0.FTCServerV1_4_0;

/**
 * 
 * 
 * @author Tecgraf
 */
public class FtcServer {

  /**
   * Configuracoes do servidor.
   */
  private static ServerConfig config = new ServerConfig();

  /**
   * Objeto responsvel por registrar as atividades do servidor.
   */
  private final static Logger logger = Logger.getLogger("tecgraf.ftc");

  /**
   * Imprime a utilizacao do programa na saida padrao.
   */
  public static void printUsage() {
    System.out
      .println("Usage: ./runJavaServer [-h host] [-p port] -f /dir/path/ ");
  }

  public static boolean isLink(final File file) throws IOException {
    if (file == null || !file.exists()) {
      return false;
    }

    File canonicalDir = file.getParentFile().getCanonicalFile();
    File fileInCanonicalDir = new File(canonicalDir, file.getName());

    return !fileInCanonicalDir.getCanonicalFile().equals(
      fileInCanonicalDir.getAbsoluteFile());
  }

  /**
   * Inicia o servidor FTC
   * 
   * @throws NoSuchAlgorithmException
   * @throws IOException
   * @throws InvalidArraySize
   * @throws MaxChannelRequestsException
   * @throws InterruptedException
   */
  public void start() throws Exception {

    String serverRoot = config.getServerRoot();

    File dir = new File(serverRoot);

    if ((serverRoot.length() == 0) || (!dir.exists()) || (!dir.isDirectory())) {
      printUsage();
      System.exit(1);
    }

    // Objeto para gerar a chave baseada no nome
    MessageDigest key = MessageDigest.getInstance("MD5");
    FTCServerVersionAdapter fileServer = getServer(config);
    DispatcherThread thread = new DispatcherThread(fileServer);
    thread.start();
    Thread.sleep(2000);

    // info output
    logger.info("OS Name: " + System.getProperty("os.name"));
    logger.info("FtcServer running on: " + config.getHostName() + ":"
      + config.getPort());
    logger.info("folder: " + dir.getCanonicalPath());

    List<File> fileList = Arrays.asList(dir.listFiles());

    // Cadastrando os arquivos
    boolean islink = false;
    for (File file : fileList) {
      islink = isLink(file);
      if (file.isFile() || islink) {
        String filename = file.getCanonicalPath();

        // Existing File
        key.update(filename.getBytes(), 0, filename.length());

        byte[] digest = key.digest();
        fileServer.createFileChannelInfo(null, filename.getBytes(), digest,
          !islink);

        BigInteger bigInt = new BigInteger(1, digest);

        logger.info(filename + ": " + bigInt.toString(16));

        key.reset();

        // Ignoring /dev/zero and /dev/null
        if (filename.contains("zero") || filename.contains("null")) {
          continue;
        }

        // Placeholder for uploading the same file
        filename += "-UPLOAD";
        key.update(filename.getBytes(), 0, filename.length());

        digest = key.digest();
        fileServer.createFileChannelInfo(null, filename.getBytes(), digest,
          !islink);

        bigInt = new BigInteger(1, digest);

        logger.info(filename + ": " + bigInt.toString(16));

        key.reset();
      }
    }
  }

  private FTCServerVersionAdapter getServer(ServerConfig config)
    throws IOException {

    FTCServerVersionAdapter rfc = null;

    switch (config.getFtcVersion()) {
      case FTC_V1_4_1:
        throw new NotImplementedException();
      case FTC_V1_4_0:
        rfc = new FTCServerV1_4_0(config);
        break;
      case FTC_V1_3:
        rfc = new FTCServerV1_3(config);
        break;
      case FTC_V1_2:
        rfc = new FTCServerV1_2(config);
        break;
      case FTC_V1_1:
      default:
        rfc = new FTCServerV1_1(config);
        break;
    }

    return rfc;
  }

  /**
   * @param args
   * @throws IOException
   * @throws NoSuchAlgorithmException
   * @throws InvalidArraySize
   * @throws MaxChannelRequestsException
   * @throws InterruptedException
   */
  public static void main(String[] args) throws Exception {

    CmdLineParser parser = new CmdLineParser(config);

    try {
      parser.parseArgument(args);
    }
    catch (CmdLineException e) {
      parser.printUsage(System.err);
      System.exit(1);
    }

    new FtcServer().start();
  }

  /**
   * Thread responsvel por iniciar o tratamento de requisies de um servidor
   * de arquivos.
   * 
   * @author Tecgraf/PUC-Rio
   */
  private static final class DispatcherThread extends Thread {
    /**
     * O servidor de arquivos.
     */
    private FTCServerVersionAdapter fileServer;

    /**
     * Cria uma thread para iniciar o tratamento de requisies de um servidor
     * de arquivos.
     * 
     * @param fileServer O servidor de arquivos.
     */
    DispatcherThread(FTCServerVersionAdapter fileServer) {
      this.fileServer = fileServer;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void run() {
      this.fileServer.dispatch();
    }
  }

}
