package ftc;

import java.util.logging.Level;

import org.kohsuke.args4j.Option;

public class ServerConfig {

  @Option(name = "-h", usage = "Nome/IP do servidor FTC.")
  private String hostname = "0.0.0.0";

  @Option(name = "-p", usage = "Porta TCP do servidor FTC.")
  private int port = 25555;

  @Option(name = "-m", usage = "Numero maximo de clientes.")
  private int maxClients = 150;

  @Option(name = "-x", usage = "Timeout do metodo 'select'.")
  private long selectTimeout = 4000;

  @Option(name = "-t", usage = "Tempo em milisegundos de espera por conexao inativa.")
  private long clientTimeout = 120000;

  @Option(name = "-b", usage = "Tamanho do buffer para cada cliente.")
  private int clientBufferSize = 1024 * 1024; // 1MB

  @Option(name = "-f", usage = "Pasta com os arquivos a serem tranferidos.", required = true)
  private String serverRoot = "";

  @Option(name = "-a", usage = "Define se o servidor deve tentar aceitar o numero maximo de clientes.")
  private boolean acceptMaxPossible = false;

  @Option(name = "-o", usage = "Nome do arquivo de saida de log.")
  private String logFilename = "/tmp/ftc-javaserver.txt";

  @Option(name = "-ftcversion", usage = "Define a verso do FTC a ser utilizada.")
  private FTCVersion ftcVersion = FTCVersion.FTC_V1_4_0;

  @Option(name = "-c", usage = "Tamanho do chunk em bytes.")
  private int chunkSize = 1024 * 1024; // 1MB

  /**
   * 
   */
  private boolean testMode = true;

  /**
   * O nvel de logging messages
   */
  private Level loglevel = Level.FINEST;

  /**
   * Tempo maximo que o servidor espera para remover uma channel nao consumido
   * em milisegundos. (Default = 6 minutos)
   */
  private long channelRequestTimeout = 360000;

  /**
   * Nmero maximo de requisicoes por canal que o cdigo cliente da biblioteca
   * pode criar sem que as requisioes sejam consumidas.
   */
  private int maxChannelRequests = 2048;

  /**
   * Retorna O nome do host para o servidor de arquivos.
   * 
   * @return hostName
   */
  public String getHostName() {
    return hostname;
  }

  /**
   * @param hostName O nome do host para o servidor de arquivos.
   */
  public void setHostName(String hostname) {
    this.hostname = hostname;
  }

  /**
   * Retorna a porta para o servidor de arquivos.
   * 
   * @return port A porta para o servidor de arquivos.
   */
  public int getPort() {
    return port;
  }

  /**
   * @param port A porta para o servidor de arquivos. Caso seja {@literal 0},
   *        uma porta disponvel ser obtida.
   */
  public void setPort(int port) {
    this.port = port;
  }

  /**
   * Retorna Tempo maximo de espera do metodo select em milisegundos.
   * 
   * @return selectTimeout
   */
  public long getSelectTimeout() {
    return selectTimeout;
  }

  /**
   * @param selectTimeout Tempo maximo de espera do metodo select em
   *        milisegundos.
   */
  public void setSelectTimeout(long selectTimeout) {
    this.selectTimeout = selectTimeout;
  }

  /**
   * Retorna
   * 
   * @return clientTimeout
   */
  public long getClientTimeout() {
    return clientTimeout;
  }

  /**
   * @param clientTimeout
   */
  public void setClientTimeout(long clientTimeout) {
    this.clientTimeout = clientTimeout;
  }

  /**
   * Retorna O numero maximo de clientes
   * 
   * @return maxClients
   */
  public int getMaxClients() {
    return maxClients;
  }

  /**
   * @param maxClients O numero maximo de clientes
   */
  public void setMaxClients(int maxClients) {
    this.maxClients = maxClients;
  }

  /**
   * Retorna Tamanho do espao de memoria utilizado na comunicaao com cada
   * cliente.
   * 
   * @return clientBufferSize
   */
  public int getClientBufferSize() {
    return clientBufferSize;
  }

  /**
   * @param clientBufferSize Define o tamanho do espao de memoria utilizado na
   *        comunicaao com cada cliente.
   */
  public void setClientBufferSize(int clientBufferSize) {
    this.clientBufferSize = clientBufferSize;
  }

  /**
   * Retora o valor da politica de aceitao de novos clientes. True para
   * aceitar o numero maximo de clientes e false para aceitar um cliente por
   * vez.
   * 
   * @return boolean
   */
  public boolean acceptMaxPossible() {
    return acceptMaxPossible;
  }

  /**
   * Define a politica de aceitao de novos clientes. True para aceitar o
   * numero maximo de clientes e false para aceitar um cliente por vez.
   * 
   * @param value
   */
  public void setAcceptMaxPossible(boolean value) {
    acceptMaxPossible = value;
  }

  /**
   * Retorna True se o servidor deve executar em modo de teste.
   * 
   * @return testMode
   */
  public boolean isTestMode() {
    return testMode;
  }

  /**
   * Define se o servidor deve executar em modo de teste.
   * 
   * @param testMode
   */
  public void setTestMode(boolean testMode) {
    this.testMode = testMode;
  }

  /**
   * @param serverRoot
   */
  public void setServerRoot(String serverRoot) {
    this.serverRoot = serverRoot;
  }

  /**
   * Retorna
   * 
   * @return serverRoot
   */
  public String getServerRoot() {
    return serverRoot;
  }

  /**
   * Define o nome do arquivo de saida do log.
   * 
   * @param filename
   */
  public void setOutputLogFilename(String filename) {
    if (filename != null) {
      this.logFilename = filename;
    }
  }

  /**
   * {@inheritDoc}
   */
  public String getOutputLogFilename() {
    return logFilename;
  }

  /**
   * {@inheritDoc}
   */
  public long getChannelRequestTimeout() {
    return channelRequestTimeout;
  }

  /**
   * {@inheritDoc}
   */
  public void setChannelRequestTimeout(long value) {
    channelRequestTimeout = value;
  }

  /**
   * {@inheritDoc}
   */
  public int getMaxChannelRequests() {
    return maxChannelRequests;
  }

  /**
   * {@inheritDoc}
   */
  public void setMaxChannelRequests(int maxChannelRequests) {
    this.maxChannelRequests = maxChannelRequests;
  }

  /**
   * {@inheritDoc}
   */
  public void setLoglevel(Level loglevel) {
    this.loglevel = loglevel;
  }

  /**
   * {@inheritDoc}
   */
  public Level getLoglevel() {
    return loglevel;
  }

  public FTCVersion getFtcVersion() {
    return ftcVersion;
  }

  public void setFtcVersion(FTCVersion ftcVersion) {
    this.ftcVersion = ftcVersion;
  }

  public int getChunkSize() {
    return this.chunkSize;
  }

  public void setChunkSize(int size) {
    this.chunkSize = size;
  }
}
